/*
 *  Copyright (C) 2009 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

/**
 * SECTION:kiba-visual-feature
 * @short_description: Base class for visual features on #KibaDock.
 * @see_also: #KibaWin,#KibaDock,#KibaObject
 *
 * Base class for visual features on #KibaDock.
 */

#ifndef __KIBA_VISUAL_FEATURE_H__
#define __KIBA_VISUAL_FEATURE_H__

#include <cairo.h>
#include <glib-object.h>

G_BEGIN_DECLS

typedef struct _KibaVisualFeatureObject KibaVisualFeatureObject;
typedef struct _KibaVisualFeature KibaVisualFeature;
typedef struct _KibaVisualFeatureClass KibaVisualFeatureClass;

#define KIBA_TYPE_VISUAL_FEATURE           (kiba_visual_feature_get_type ())
#define KIBA_VISUAL_FEATURE(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_VISUAL_FEATURE, KibaVisualFeature))
#define KIBA_VISUAL_FEATURE_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_TYPE_VISUAL_FEATURE,  KibaVisualFeatureClass))
#define KIBA_IS_VISUAL_FEATURE(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_VISUAL_FEATURE))
#define KIBA_IS_VISUAL_FEATURE_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_VISUAL_FEATURE))
#define KIBA_VISUAL_FEATURE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_VISUAL_FEATURE, KibaVisualFeatureClass))


typedef enum {
	KIBA_VISUAL_FEATURE_FIRST = 0,
	KIBA_VISUAL_FEATURE_MORPH,
	KIBA_VISUAL_FEATURE_ONTOP,
	KIBA_VISUAL_FEATURE_GIMMICK,
	KIBA_VISUAL_FEATURE_ONTOP_OF_DOCK,
	KIBA_VISUAL_FEATURE_LAST
}KibaVisualFeatureType;

#include <kiba-object.h>
#include <kiba-win.h>

struct _KibaVisualFeatureObject {
	KibaVisualFeature *feature;
	
	KibaObject *object;
	KibaAllocation allocation; /**< allocation relative to object */
	KibaAllocation abs_allocation; /**< allocation relative to window */
	gboolean queue_resize;
};

struct _KibaVisualFeature {
	GObject parent;
	
	/* list of KibaVisualFeatureObject's */
	GList *objects;
	
	/* overridable methods */
	void (*win_set) (KibaVisualFeature*, KibaWin *win);
	void (*object_added) (KibaVisualFeature*, KibaObject* object);
	void (*object_removed) (KibaVisualFeature*, KibaObject* object);
	void (*draw) (KibaVisualFeature*, KibaVisualFeatureObject*, cairo_t*);
	/* should be fast, better precalculate stuff */
	KibaAllocation (* get_allocation) (KibaVisualFeature*, KibaVisualFeatureObject*);
	/* should be fast, better precalculate stuff */
	void (* get_max_offset) (KibaVisualFeature*,
		KibaVisualFeatureObject *object,
		float *left, float *right,
		float *top, float *bottom);
};

struct _KibaVisualFeatureClass {
	GObjectClass parent_class;
};

#include <kiba-win.h>

const char* kiba_visual_feature_get_name (KibaVisualFeature*);

KibaVisualFeatureType kiba_visual_feature_type (KibaVisualFeature*);

gboolean kiba_visual_feature_get_front (KibaVisualFeature *);


void kiba_visual_feature_update_allocation(KibaVisualFeature*, KibaVisualFeatureObject *);

void kiba_visual_feature_queue_draw(KibaVisualFeature*, KibaVisualFeatureObject*);

void kiba_visual_feature_queue_resize(KibaVisualFeature*, KibaVisualFeatureObject*);

void kiba_visual_feature_queue_draw_all(KibaVisualFeature*);

void kiba_visual_feature_queue_resize_all(KibaVisualFeature*);


KibaVisualFeatureObject* kiba_visual_feature_add_object(KibaVisualFeature*, KibaObject*);

void kiba_visual_feature_remove_object(KibaVisualFeature*, KibaObject*);


void kiba_visual_feature_object_added (KibaVisualFeature*, KibaObject*);

void kiba_visual_feature_object_removed (KibaVisualFeature*, KibaObject*);

void kiba_visual_feature_draw(KibaVisualFeature*, KibaVisualFeatureObject*, cairo_t *);

KibaDock* kiba_visual_feature_get_dock(KibaVisualFeature *feature);

KibaWin* kiba_visual_feature_get_win(KibaVisualFeature *feature);

void kiba_visual_feature_set_win(KibaVisualFeature *feature, KibaWin* win);

void
kiba_visual_feature_do_resize(KibaVisualFeature *feature,
		KibaVisualFeatureObject *object);

void kiba_visual_feature_translate_to_dnd_window(
		KibaVisualFeature *feature,
		KibaVisualFeatureObject *object,
		KibaAllocation *alloc,
		float *offset_x,
		float *offset_y);

KibaVisualFeature* kiba_visual_feature_new(
	KibaWin *win,
	const char *name,
	KibaVisualFeatureType type,
	gboolean front);

G_END_DECLS

#endif
